package org.archive.search.example;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.PrintWriter;
import java.io.StringReader;
import java.io.StringWriter;
import java.net.URL;
import java.net.URLConnection;

import javax.servlet.ServletContext;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.xml.transform.Source;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.stream.StreamResult;
import javax.xml.transform.stream.StreamSource;

/**
 * This Servlet is a custom Java Web application.  It queries the OpenSearch server hosted by
 * Archive-It and transforms the XML results into an HTML user interface
 * NOTE: This is a SAMPLE application and should not be used in a production environment
 */
public class SearchResultGenerator extends HttpServlet {
	private static final long serialVersionUID = 1L;

	// default OpenSearch host
	String searchHost = "archive-it.org";
	
	// default OpenSearch URL
	String searchPath = "/seam/resource/opensearch";
	
    /**
     * @see HttpServlet#HttpServlet()
     */
    public SearchResultGenerator() {
        super();
        // TODO Auto-generated constructor stub
    }

	/**
	 * @see HttpServlet#doGet(HttpServletRequest request, HttpServletResponse response)
	 */
	protected void doGet(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException {

		// get the value of the "q" parameter from the URL's query string
		// the "q" parameter value is the search phrase
	    String searchPhrase = request.getParameter("q");
	    
	    // get the values of the "i" parameter from the URL's query string
	    // the "i" parameter's values are the collection numbers to search
	    String[] iParamValues = request.getParameterValues("i");
	    
	    // get the values of the "t" parameter from the URL's query string
	    // the "t" parameter's values specify the file types to retrieve	   
	    String[] tParamValues = request.getParameterValues("t");

	    // sanity check
	    if (searchPhrase == null) searchPhrase = "";
	    
	    // do search query and retrieve XML response from Archive-It OpenSearch server
	    String xmlResponse = doSearchQuery(searchPhrase, iParamValues, tParamValues);
	    
	    // transform the XML response into HTML using XSL style sheet
	    String html = null;
	    
	    try {
	    	html = transformXMLtoHTML(xmlResponse);
	    }
	    catch (Exception exc) {
	    	exc.printStackTrace();
	    	html = "error";
	    }
	    
	    // send HTML back to browser	    
	    response.setContentType("text/html");
	    PrintWriter out = response.getWriter();
	    
	    out.println(html);		
	}

	/**
	 * Transform the XML response from the OpenSearch server into an HTML user interface
	 * @param xmlResponse
	 * @return The search result formatted as XML
	 * @throws Exception
	 */
	private String transformXMLtoHTML(String xmlResponse) throws Exception {
		
	    TransformerFactory tFactory = TransformerFactory.newInstance();

	    // put your custom style sheet's name here
	    //String filename = "/WEB-INF/search.xsl";
	    String filename = "/WEB-INF/search_title.xsl";	    
	    //String filename = "/WEB-INF/search_collections.xsl";
	    
	    ServletContext context = getServletContext();	    
	    InputStream inputStream = context.getResourceAsStream(filename);	    
	    
	    Source xsltSource = new StreamSource(inputStream);
	    Source xmlSource = new StreamSource(new StringReader(xmlResponse));
	    
	    StreamResult htmlOutput = new StreamResult(new StringWriter());	    
	    
        Transformer trans = tFactory.newTransformer(xsltSource);
        
        trans.transform(xmlSource, htmlOutput);
	    
	    return htmlOutput.getWriter().toString();
	}

	/**
	 * Execute the search against the OpenSearch server and retrieve the XML search result response
	 * @param searchPhrase
	 * @param collectionIds
	 * @param docTypes
	 * @return
	 */
	private String doSearchQuery(String searchPhrase, String [] collectionIds, String [] docTypes) {

		//the URL to submit to the OpenSearch server
		StringBuffer openSearchQueryString = new StringBuffer();
		
		// add the necessary parameters to the URL's query string
		openSearchQueryString.append("?q=");
		openSearchQueryString.append(searchPhrase);
		
		if (collectionIds != null && collectionIds.length > 0) {
			
			for (int i = 0; i < collectionIds.length; ++i) {
				openSearchQueryString.append("&i=");
				openSearchQueryString.append(collectionIds[i]);	
			}
			
		}
		
		if (docTypes != null && docTypes.length > 0) {
			
			for (int i = 0; i < docTypes.length; ++i) {
				openSearchQueryString.append("&t=");
				openSearchQueryString.append(docTypes[i]);	
			}			
		}		
		
		// as a debugging step, print the open search URL to be submitted
		// TODO: comment this line out for production
		System.out.println(openSearchQueryString.toString());
		
		String xmlResponse = null;
		
		// get XML response from OpenSearch Server
		try {
			xmlResponse = sendQuery(openSearchQueryString.toString());
		}
		catch (Exception exc) {
			exc.printStackTrace();
			return null;
		}
		
		return xmlResponse;		
	}

	/**
	 * Open a network connection to the OpenSearch server and submit the OpenSearch query URL
	 * @param openSearchQuery
	 * @return
	 * @throws Exception
	 */
	private String sendQuery(String openSearchQuery) throws Exception {
		
        URL searcher = new URL("http://" + searchHost + searchPath + openSearchQuery);
        URLConnection searcherConnection = searcher.openConnection();
        
        BufferedReader in = new BufferedReader(
                                new InputStreamReader(
                                searcherConnection.getInputStream()));
        
        StringBuffer searchResultXMLResponse = new StringBuffer();
        String inputLine;        

        while ((inputLine = in.readLine()) != null) 
        	searchResultXMLResponse.append(inputLine);          
        
        in.close();
        
        return searchResultXMLResponse.toString();
	}

	/**
	 * @see HttpServlet#doPost(HttpServletRequest request, HttpServletResponse response)
	 */
	protected void doPost(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException {
		// TODO Auto-generated method stub
	}

}
